package solutions.bitbadger.documents.core.tests.java;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.AutoId;
import solutions.bitbadger.documents.Configuration;
import solutions.bitbadger.documents.Dialect;
import solutions.bitbadger.documents.DocumentException;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * Unit tests for the `Configuration` object
 */
@DisplayName("Core | Java | Configuration")
final public class ConfigurationTest {

    @Test
    @DisplayName("Default ID field is `id`")
    public void defaultIdField() {
        assertEquals("id", Configuration.idField, "Default ID field incorrect");
    }

    @Test
    @DisplayName("Default Auto ID strategy is `DISABLED`")
    public void defaultAutoId() {
        assertEquals(AutoId.DISABLED, Configuration.autoIdStrategy, "Default Auto ID strategy should be `disabled`");
    }

    @Test
    @DisplayName("Default ID string length should be 16")
    public void defaultIdStringLength() {
        assertEquals(16, Configuration.idStringLength, "Default ID string length should be 16");
    }

    @Test
    @DisplayName("Dialect is derived from connection string")
    public void dialectIsDerived() throws DocumentException {
        try {
            assertThrows(DocumentException.class, Configuration::dialect);
            Configuration.setConnectionString("jdbc:postgresql:db");
            assertEquals(Dialect.POSTGRESQL, Configuration.dialect());
        } finally {
            Configuration.setConnectionString(null);
        }
    }
}
