package solutions.bitbadger.documents.core.tests.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.query.PatchQuery;
import solutions.bitbadger.documents.core.tests.ForceDialect;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;

/**
 * Unit tests for the `Patch` object
 */
@DisplayName("Core | Java | Query | PatchQuery")
final public class PatchQueryTest {

    /**
     * Reset the dialect
     */
    @AfterEach
    public void cleanUp() {
        ForceDialect.none();
    }

    @Test
    @DisplayName("byId generates correctly | PostgreSQL")
    public void byIdPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("UPDATE %s SET data = data || :data WHERE data->>'id' = :id", TEST_TABLE),
                PatchQuery.byId(TEST_TABLE), "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byId generates correctly | SQLite")
    public void byIdSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format(
                        "UPDATE %s SET data = json_patch(data, json(:data)) WHERE data->>'id' = :id", TEST_TABLE),
                PatchQuery.byId(TEST_TABLE), "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | PostgreSQL")
    public void byFieldsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("UPDATE %s SET data = data || :data WHERE data->>'z' = :y", TEST_TABLE),
                PatchQuery.byFields(TEST_TABLE, List.of(Field.equal("z", "", ":y"))),
                "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byFields generates correctly | SQLite")
    public void byFieldsSQLite() throws DocumentException {
        ForceDialect.sqlite();
        assertEquals(String.format(
                        "UPDATE %s SET data = json_patch(data, json(:data)) WHERE data->>'z' = :y", TEST_TABLE),
                PatchQuery.byFields(TEST_TABLE, List.of(Field.equal("z", "", ":y"))),
                "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byContains generates correctly | PostgreSQL")
    public void byContainsPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("UPDATE %s SET data = data || :data WHERE data @> :criteria", TEST_TABLE),
                PatchQuery.byContains(TEST_TABLE), "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byContains fails | SQLite")
    public void byContainsSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> PatchQuery.byContains(TEST_TABLE));
    }

    @Test
    @DisplayName("byJsonPath generates correctly | PostgreSQL")
    public void byJsonPathPostgres() throws DocumentException {
        ForceDialect.postgres();
        assertEquals(String.format("UPDATE %s SET data = data || :data WHERE jsonb_path_exists(data, :path::jsonpath)",
                        TEST_TABLE),
                PatchQuery.byJsonPath(TEST_TABLE), "Patch query not constructed correctly");
    }

    @Test
    @DisplayName("byJsonPath fails | SQLite")
    public void byJsonPathSQLite() {
        ForceDialect.sqlite();
        assertThrows(DocumentException.class, () -> PatchQuery.byJsonPath(TEST_TABLE));
    }
}
