package solutions.bitbadger.documents.core.tests.java.integration;

import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.Field;
import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;
import static solutions.bitbadger.documents.java.extensions.ConnExt.*;

/**
 * Integration tests for the `Count` object
 */
final public class CountFunctions {

    public static void all(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(5L, countAll(db.getConn(), TEST_TABLE), "There should have been 5 documents in the table");
    }

    public static void byFieldsNumeric(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(3L, countByFields(db.getConn(), TEST_TABLE, List.of(Field.between("numValue", 10, 20))),
                "There should have been 3 matching documents");
    }

    public static void byFieldsAlpha(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(1L, countByFields(db.getConn(), TEST_TABLE, List.of(Field.between("value", "aardvark", "apple"))),
                "There should have been 1 matching document");
    }

    public static void byContainsMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(2L, countByContains(db.getConn(), TEST_TABLE, Map.of("value", "purple")),
                "There should have been 2 matching documents");
    }

    public static void byContainsNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(0L, countByContains(db.getConn(), TEST_TABLE, Map.of("value", "magenta")),
                "There should have been no matching documents");
    }

    public static void byJsonPathMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(2L, countByJsonPath(db.getConn(), TEST_TABLE, "$.numValue ? (@ < 5)"),
                "There should have been 2 matching documents");
    }

    public static void byJsonPathNoMatch(ThrowawayDatabase db) throws DocumentException {
        JsonDocument.load(db);
        assertEquals(0L, countByJsonPath(db.getConn(), TEST_TABLE, "$.numValue ? (@ > 100)"),
                "There should have been no matching documents");
    }

    private CountFunctions() {
    }
}
