package solutions.bitbadger.documents.core.tests.java.integration;

import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.java.Document;
import solutions.bitbadger.documents.core.tests.integration.ThrowawayDatabase;

import java.util.List;

import static org.junit.jupiter.api.Assertions.fail;
import static solutions.bitbadger.documents.core.tests.TypesKt.TEST_TABLE;

public class JsonDocument {

    private String id;
    private String value;
    private int numValue;
    private SubDocument sub;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public int getNumValue() {
        return numValue;
    }

    public void setNumValue(int numValue) {
        this.numValue = numValue;
    }

    public SubDocument getSub() {
        return sub;
    }

    public void setSub(SubDocument sub) {
        this.sub = sub;
    }

    public JsonDocument(String id, String value, int numValue, SubDocument sub) {
        this.id = id;
        this.value = value;
        this.numValue = numValue;
        this.sub = sub;
    }

    public JsonDocument(String id, String value, int numValue) {
        this(id, value, numValue, null);
    }

    public JsonDocument(String id) {
        this(id, "", 0, null);
    }
    
    public JsonDocument() {
        this(null);
    }

    private static final List<JsonDocument> testDocuments = List.of(
            new JsonDocument("one", "FIRST!", 0),
            new JsonDocument("two", "another", 10, new SubDocument("green", "blue")),
            new JsonDocument("three", "", 4),
            new JsonDocument("four", "purple", 17, new SubDocument("green", "red")),
            new JsonDocument("five", "purple", 18));

    public static void load(ThrowawayDatabase db, String tableName) {
        try {
            for (JsonDocument doc : testDocuments) {
                Document.insert(tableName, doc, db.getConn());
            }
        } catch (DocumentException ex) {
            fail("Could not load test documents", ex);
        }
    }

    public static void load(ThrowawayDatabase db) {
        load(db, TEST_TABLE);
    }

    /** Document ID <code>one</code> as a JSON string */
    public static String one = "{\"id\":\"one\",\"value\":\"FIRST!\",\"numValue\":0,\"sub\":null}";

    /** Document ID <code>two</code> as a JSON string */
    public static String two = "{\"id\":\"two\",\"value\":\"another\",\"numValue\":10,"
            + "\"sub\":{\"foo\":\"green\",\"bar\":\"blue\"}}";

    /** Document ID <code>three</code> as a JSON string */
    public static String three = "{\"id\":\"three\",\"value\":\"\",\"numValue\":4,\"sub\":null}";

    /** Document ID <code>four</code> as a JSON string */
    public static String four = "{\"id\":\"four\",\"value\":\"purple\",\"numValue\":17,"
            + "\"sub\":{\"foo\":\"green\",\"bar\":\"red\"}}";

    /** Document ID <code>five</code> as a JSON string */
    public static String five = "{\"id\":\"five\",\"value\":\"purple\",\"numValue\":18,\"sub\":null}";
}
