package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.PgDB;

/**
 * PostgreSQL integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName("Core | Java | PostgreSQL: RemoveFields")
final public class PostgreSQLRemoveFieldsIT {

    @Test
    @DisplayName("byId removes fields from an existing document")
    public void byIdMatchFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byIdMatchFields(db);
        }
    }

    @Test
    @DisplayName("byId succeeds when fields do not exist on an existing document")
    public void byIdMatchNoFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byIdMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byId succeeds when no document exists")
    public void byIdNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byIdNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields removes fields from matching documents")
    public void byFieldsMatchFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byFieldsMatchFields(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when fields do not exist on matching documents")
    public void byFieldsMatchNoFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byFieldsMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no matching documents exist")
    public void byFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byContains removes fields from matching documents")
    public void byContainsMatchFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byContainsMatchFields(db);
        }
    }

    @Test
    @DisplayName("byContains succeeds when fields do not exist on matching documents")
    public void byContainsMatchNoFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byContainsMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byContains succeeds when no matching documents exist")
    public void byContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath removes fields from matching documents")
    public void byJsonPathMatchFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byJsonPathMatchFields(db);
        }
    }

    @Test
    @DisplayName("byJsonPath succeeds when fields do not exist on matching documents")
    public void byJsonPathMatchNoFields() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byJsonPathMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byJsonPath succeeds when no matching documents exist")
    public void byJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            RemoveFieldsFunctions.byJsonPathNoMatch(db);
        }
    }
}
