package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Definition` object / `ensure*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Definition")
final public class SQLiteDefinitionIT {

    @Test
    @DisplayName("ensureTable creates table and index")
    public void ensureTable() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DefinitionFunctions.ensuresATable(db);
        }
    }

    @Test
    @DisplayName("ensureFieldIndex creates an index")
    public void ensureFieldIndex() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            DefinitionFunctions.ensuresAFieldIndex(db);
        }
    }

    @Test
    @DisplayName("ensureDocumentIndex fails for full index")
    public void ensureDocumentIndexFull() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> DefinitionFunctions.ensureDocumentIndexFull(db));
        }
    }

    @Test
    @DisplayName("ensureDocumentIndex fails for optimized index")
    public void ensureDocumentIndexOptimized() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> DefinitionFunctions.ensureDocumentIndexOptimized(db));
        }
    }
}
