package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Find")
final public class SQLiteFindIT {

    @Test
    @DisplayName("all retrieves all documents")
    public void allDefault() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.allDefault(db);
        }
    }

    @Test
    @DisplayName("all sorts data ascending")
    public void allAscending() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.allAscending(db);
        }
    }

    @Test
    @DisplayName("all sorts data descending")
    public void allDescending() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.allDescending(db);
        }
    }

    @Test
    @DisplayName("all sorts data numerically")
    public void allNumOrder() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.allNumOrder(db);
        }
    }

    @Test
    @DisplayName("all succeeds with an empty table")
    public void allEmpty() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.allEmpty(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    public void byIdString() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byIdString(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    public void byIdNumber() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byIdNumber(db);
        }
    }

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    public void byIdNotFound() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byIdNotFound(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    public void byFieldsMatchOrdered() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    public void byFieldsMatchNumIn() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    public void byFieldsMatchInArray() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    public void byFieldsNoMatchInArray() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.byFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> FindFunctions.byContainsMatch(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> FindFunctions.byJsonPathMatch(db));
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    public void firstByFieldsMatchOne() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.firstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    public void firstByFieldsMatchMany() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.firstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    public void firstByFieldsMatchOrdered() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.firstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    public void firstByFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            FindFunctions.firstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByContains fails")
    public void firstByContainsFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> FindFunctions.firstByContainsMatchOne(db));
        }
    }

    @Test
    @DisplayName("firstByJsonPath fails")
    public void firstByJsonPathFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> FindFunctions.firstByJsonPathMatchOne(db));
        }
    }
}
