package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: RemoveFields")
final public class SQLiteRemoveFieldsIT {

    @Test
    @DisplayName("byId removes fields from an existing document")
    public void byIdMatchFields() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byIdMatchFields(db);
        }
    }

    @Test
    @DisplayName("byId succeeds when fields do not exist on an existing document")
    public void byIdMatchNoFields() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byIdMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byId succeeds when no document exists")
    public void byIdNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byIdNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields removes fields from matching documents")
    public void byFieldsMatchFields() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byFieldsMatchFields(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when fields do not exist on matching documents")
    public void byFieldsMatchNoFields() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byFieldsMatchNoFields(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no matching documents exist")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            RemoveFieldsFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> RemoveFieldsFunctions.byContainsMatchFields(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathMatch() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> RemoveFieldsFunctions.byJsonPathMatchFields(db));
        }
    }
}
