package solutions.bitbadger.documents.core.tests.integration

import org.junit.jupiter.api.DisplayName
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName("Core | Kotlin | PostgreSQL: Document")
class PostgreSQLDocumentIT {

    @Test
    @DisplayName("insert works with default values")
    fun insertDefault() =
        PgDB().use(DocumentFunctions::insertDefault)

    @Test
    @DisplayName("insert fails with duplicate key")
    fun insertDupe() =
        PgDB().use(DocumentFunctions::insertDupe)

    @Test
    @DisplayName("insert succeeds with numeric auto IDs")
    fun insertNumAutoId() =
        PgDB().use(DocumentFunctions::insertNumAutoId)

    @Test
    @DisplayName("insert succeeds with UUID auto ID")
    fun insertUUIDAutoId() =
        PgDB().use(DocumentFunctions::insertUUIDAutoId)

    @Test
    @DisplayName("insert succeeds with random string auto ID")
    fun insertStringAutoId() =
        PgDB().use(DocumentFunctions::insertStringAutoId)

    @Test
    @DisplayName("save updates an existing document")
    fun saveMatch() =
        PgDB().use(DocumentFunctions::saveMatch)

    @Test
    @DisplayName("save inserts a new document")
    fun saveNoMatch() =
        PgDB().use(DocumentFunctions::saveNoMatch)

    @Test
    @DisplayName("update replaces an existing document")
    fun updateMatch() =
        PgDB().use(DocumentFunctions::updateMatch)

    @Test
    @DisplayName("update succeeds when no document exists")
    fun updateNoMatch() =
        PgDB().use(DocumentFunctions::updateNoMatch)
}
