package solutions.bitbadger.documents.core.tests.integration

import org.junit.jupiter.api.DisplayName
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Core | Kotlin | PostgreSQL: Json")
class PostgreSQLJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    fun allDefault() =
        PgDB().use(JsonFunctions::allDefault)

    @Test
    @DisplayName("all succeeds with an empty table")
    fun allEmpty() =
        PgDB().use(JsonFunctions::allEmpty)

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    fun byIdString() =
        PgDB().use(JsonFunctions::byIdString)

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    fun byIdNumber() =
        PgDB().use(JsonFunctions::byIdNumber)

    @Test
    @DisplayName("byId returns an empty document when a matching ID is not found")
    fun byIdNotFound() =
        PgDB().use(JsonFunctions::byIdNotFound)

    @Test
    @DisplayName("byFields retrieves matching documents")
    fun byFieldsMatch() =
        PgDB().use(JsonFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    fun byFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::byFieldsMatchOrdered)

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    fun byFieldsMatchNumIn() =
        PgDB().use(JsonFunctions::byFieldsMatchNumIn)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        PgDB().use(JsonFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    fun byFieldsMatchInArray() =
        PgDB().use(JsonFunctions::byFieldsMatchInArray)

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    fun byFieldsNoMatchInArray() =
        PgDB().use(JsonFunctions::byFieldsNoMatchInArray)

    @Test
    @DisplayName("byContains retrieves matching documents")
    fun byContainsMatch() =
        PgDB().use(JsonFunctions::byContainsMatch)

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    fun byContainsMatchOrdered() =
        PgDB().use(JsonFunctions::byContainsMatchOrdered)

    @Test
    @DisplayName("byContains succeeds when no documents match")
    fun byContainsNoMatch() =
        PgDB().use(JsonFunctions::byContainsNoMatch)

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    fun byJsonPathMatch() =
        PgDB().use(JsonFunctions::byJsonPathMatch)

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    fun byJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::byJsonPathMatchOrdered)

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    fun byJsonPathNoMatch() =
        PgDB().use(JsonFunctions::byJsonPathNoMatch)

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    fun firstByFieldsMatchOne() =
        PgDB().use(JsonFunctions::firstByFieldsMatchOne)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    fun firstByFieldsMatchMany() =
        PgDB().use(JsonFunctions::firstByFieldsMatchMany)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    fun firstByFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::firstByFieldsMatchOrdered)

    @Test
    @DisplayName("firstByFields returns an empty document when no document matches")
    fun firstByFieldsNoMatch() =
        PgDB().use(JsonFunctions::firstByFieldsNoMatch)

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    fun firstByContainsMatchOne() =
        PgDB().use(JsonFunctions::firstByContainsMatchOne)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    fun firstByContainsMatchMany() =
        PgDB().use(JsonFunctions::firstByContainsMatchMany)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    fun firstByContainsMatchOrdered() =
        PgDB().use(JsonFunctions::firstByContainsMatchOrdered)

    @Test
    @DisplayName("firstByContains returns an empty document when no document matches")
    fun firstByContainsNoMatch() =
        PgDB().use(JsonFunctions::firstByContainsNoMatch)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    fun firstByJsonPathMatchOne() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchOne)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    fun firstByJsonPathMatchMany() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchMany)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    fun firstByJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchOrdered)

    @Test
    @DisplayName("firstByJsonPath returns an empty document when no document matches")
    fun firstByJsonPathNoMatch() =
        PgDB().use(JsonFunctions::firstByJsonPathNoMatch)

    @Test
    @DisplayName("writeAll retrieves all documents")
    fun writeAllDefault() =
        PgDB().use(JsonFunctions::writeAllDefault)

    @Test
    @DisplayName("writeAll succeeds with an empty table")
    fun writeAllEmpty() =
        PgDB().use(JsonFunctions::writeAllEmpty)

    @Test
    @DisplayName("writeById retrieves a document via a string ID")
    fun writeByIdString() =
        PgDB().use(JsonFunctions::writeByIdString)

    @Test
    @DisplayName("writeById retrieves a document via a numeric ID")
    fun writeByIdNumber() =
        PgDB().use(JsonFunctions::writeByIdNumber)

    @Test
    @DisplayName("writeById writes an empty document when a matching ID is not found")
    fun writeByIdNotFound() =
        PgDB().use(JsonFunctions::writeByIdNotFound)

    @Test
    @DisplayName("writeByFields retrieves matching documents")
    fun writeByFieldsMatch() =
        PgDB().use(JsonFunctions::writeByFieldsMatch)

    @Test
    @DisplayName("writeByFields retrieves ordered matching documents")
    fun writeByFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::writeByFieldsMatchOrdered)

    @Test
    @DisplayName("writeByFields retrieves matching documents with a numeric IN clause")
    fun writeByFieldsMatchNumIn() =
        PgDB().use(JsonFunctions::writeByFieldsMatchNumIn)

    @Test
    @DisplayName("writeByFields succeeds when no documents match")
    fun writeByFieldsNoMatch() =
        PgDB().use(JsonFunctions::writeByFieldsNoMatch)

    @Test
    @DisplayName("writeByFields retrieves matching documents with an IN_ARRAY comparison")
    fun writeByFieldsMatchInArray() =
        PgDB().use(JsonFunctions::writeByFieldsMatchInArray)

    @Test
    @DisplayName("writeByFields succeeds when no documents match an IN_ARRAY comparison")
    fun writeByFieldsNoMatchInArray() =
        PgDB().use(JsonFunctions::writeByFieldsNoMatchInArray)

    @Test
    @DisplayName("writeByContains retrieves matching documents")
    fun writeByContainsMatch() =
        PgDB().use(JsonFunctions::writeByContainsMatch)

    @Test
    @DisplayName("writeByContains retrieves ordered matching documents")
    fun writeByContainsMatchOrdered() =
        PgDB().use(JsonFunctions::writeByContainsMatchOrdered)

    @Test
    @DisplayName("writeByContains succeeds when no documents match")
    fun writeByContainsNoMatch() =
        PgDB().use(JsonFunctions::writeByContainsNoMatch)

    @Test
    @DisplayName("writeByJsonPath retrieves matching documents")
    fun writeByJsonPathMatch() =
        PgDB().use(JsonFunctions::writeByJsonPathMatch)

    @Test
    @DisplayName("writeByJsonPath retrieves ordered matching documents")
    fun writeByJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::writeByJsonPathMatchOrdered)

    @Test
    @DisplayName("writeByJsonPath succeeds when no documents match")
    fun writeByJsonPathNoMatch() =
        PgDB().use(JsonFunctions::writeByJsonPathNoMatch)

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document")
    fun writeFirstByFieldsMatchOne() =
        PgDB().use(JsonFunctions::writeFirstByFieldsMatchOne)

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many")
    fun writeFirstByFieldsMatchMany() =
        PgDB().use(JsonFunctions::writeFirstByFieldsMatchMany)

    @Test
    @DisplayName("writeFirstByFields retrieves a matching document among many (ordered)")
    fun writeFirstByFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::writeFirstByFieldsMatchOrdered)

    @Test
    @DisplayName("writeFirstByFields writes an empty document when no document matches")
    fun writeFirstByFieldsNoMatch() =
        PgDB().use(JsonFunctions::writeFirstByFieldsNoMatch)

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document")
    fun writeFirstByContainsMatchOne() =
        PgDB().use(JsonFunctions::writeFirstByContainsMatchOne)

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document among many")
    fun writeFirstByContainsMatchMany() =
        PgDB().use(JsonFunctions::writeFirstByContainsMatchMany)

    @Test
    @DisplayName("writeFirstByContains retrieves a matching document among many (ordered)")
    fun writeFirstByContainsMatchOrdered() =
        PgDB().use(JsonFunctions::writeFirstByContainsMatchOrdered)

    @Test
    @DisplayName("writeFirstByContains writes an empty document when no document matches")
    fun writeFirstByContainsNoMatch() =
        PgDB().use(JsonFunctions::writeFirstByContainsNoMatch)

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document")
    fun writeFirstByJsonPathMatchOne() =
        PgDB().use(JsonFunctions::writeFirstByJsonPathMatchOne)

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document among many")
    fun writeFirstByJsonPathMatchMany() =
        PgDB().use(JsonFunctions::writeFirstByJsonPathMatchMany)

    @Test
    @DisplayName("writeFirstByJsonPath retrieves a matching document among many (ordered)")
    fun writeFirstByJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::writeFirstByJsonPathMatchOrdered)

    @Test
    @DisplayName("writeFirstByJsonPath writes an empty document when no document matches")
    fun writeFirstByJsonPathNoMatch() =
        PgDB().use(JsonFunctions::writeFirstByJsonPathNoMatch)
}
