package solutions.bitbadger.documents.core.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Patch` object / `patchBy*` connection extension functions
 */
@DisplayName("Core | Kotlin | SQLite: Patch")
class SQLitePatchIT {

    @Test
    @DisplayName("byId patches an existing document")
    fun byIdMatch() =
        SQLiteDB().use(PatchFunctions::byIdMatch)

    @Test
    @DisplayName("byId succeeds for a non-existent document")
    fun byIdNoMatch() =
        SQLiteDB().use(PatchFunctions::byIdNoMatch)

    @Test
    @DisplayName("byFields patches matching document")
    fun byFieldsMatch() =
        SQLiteDB().use(PatchFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        SQLiteDB().use(PatchFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byContains fails")
    fun byContainsFails() {
        assertThrows<DocumentException> { SQLiteDB().use(PatchFunctions::byContainsMatch) }
    }

    @Test
    @DisplayName("byJsonPath fails")
    fun byJsonPathFails() {
        assertThrows<DocumentException> { SQLiteDB().use(PatchFunctions::byJsonPathMatch) }
    }
}