package solutions.bitbadger.documents.groovy.tests

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.PatchQuery

import static Types.TEST_TABLE
import static org.junit.jupiter.api.Assertions.*

/**
 * Unit tests for the `Patch` object
 */
@DisplayName('Groovy | Query | PatchQuery')
class PatchQueryTest {

    /**
     * Reset the dialect
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('byId generates correctly | PostgreSQL')
    void byIdPostgres() {
        ForceDialect.postgres()
        assertEquals("UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'id' = :id".toString(),
                PatchQuery.byId(TEST_TABLE), 'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byId generates correctly | SQLite')
    void byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals("UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'id' = :id".toString(),
                PatchQuery.byId(TEST_TABLE), 'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals("UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'z' = :y".toString(),
                PatchQuery.byFields(TEST_TABLE, List.of(Field.equal('z', '', ':y'))),
                'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals("UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'z' = :y".toString(),
                PatchQuery.byFields(TEST_TABLE, List.of(Field.equal('z', '', ':y'))),
                'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals("UPDATE $TEST_TABLE SET data = data || :data WHERE data @> :criteria".toString(),
                PatchQuery.byContains(TEST_TABLE), 'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byContains fails | SQLite')
    void byContainsSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { PatchQuery.byContains(TEST_TABLE) }
    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals(
                "UPDATE $TEST_TABLE SET data = data || :data WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                PatchQuery.byJsonPath(TEST_TABLE), 'Patch query not constructed correctly')
    }

    @Test
    @DisplayName('byJsonPath fails | SQLite')
    void byJsonPathSQLite() {
        ForceDialect.sqlite()
        assertThrows(DocumentException) { PatchQuery.byJsonPath(TEST_TABLE) }
    }
}
