package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.Field

import static org.junit.jupiter.api.Assertions.*
import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

final class ExistsFunctions {

    static void byIdMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertTrue db.conn.existsById(TEST_TABLE, 'three'), 'The document with ID "three" should exist'
    }

    static void byIdNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertFalse db.conn.existsById(TEST_TABLE, 'seven'), 'The document with ID "seven" should not exist'
    }

    static void byFieldsMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertTrue(db.conn.existsByFields(TEST_TABLE, List.of(Field.equal('numValue', 10))),
                'Matching documents should have been found')
    }

    static void byFieldsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertFalse(db.conn.existsByFields(TEST_TABLE, List.of(Field.equal('nothing', 'none'))),
                'No matching documents should have been found')
    }

    static void byContainsMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertTrue(db.conn.existsByContains(TEST_TABLE, Map.of('value', 'purple')),
                'Matching documents should have been found')
    }

    static void byContainsNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertFalse(db.conn.existsByContains(TEST_TABLE, Map.of('value', 'violet')),
                'Matching documents should not have been found')
    }

    static void byJsonPathMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertTrue(db.conn.existsByJsonPath(TEST_TABLE, '$.numValue ? (@ == 10)'),
                'Matching documents should have been found')
    }

    static void byJsonPathNoMatch(ThrowawayDatabase db) {
        JsonDocument.load db
        assertFalse(db.conn.existsByJsonPath(TEST_TABLE, '$.numValue ? (@ == 10.1)'),
                'Matching documents should not have been found')
    }
}
