package solutions.bitbadger.documents.groovy.tests.integration

import solutions.bitbadger.documents.Configuration
import solutions.bitbadger.documents.Parameter
import solutions.bitbadger.documents.ParameterType
import solutions.bitbadger.documents.java.DocumentConfig
import solutions.bitbadger.documents.java.Results

import static solutions.bitbadger.documents.groovy.tests.Types.TEST_TABLE

/**
 * A wrapper for a throwaway PostgreSQL database
 */
class PgDB implements ThrowawayDatabase {

    PgDB() {
        Configuration.setConnectionString connString('postgres')
        Configuration.dbConn().withCloseable { it.customNonQuery "CREATE DATABASE $dbName" }

        Configuration.setConnectionString connString(dbName)
        conn = Configuration.dbConn()
        conn.ensureTable TEST_TABLE

        // Use a Jackson-based document serializer for testing
        DocumentConfig.serializer = new JacksonDocumentSerializer()
    }

    void close() {
        conn.close()
        Configuration.setConnectionString connString('postgres')
        Configuration.dbConn().withCloseable { it.customNonQuery "DROP DATABASE $dbName" }
        Configuration.setConnectionString null
    }

    boolean dbObjectExists(String name) {
        conn.customScalar('SELECT EXISTS (SELECT 1 FROM pg_class WHERE relname = :name) AS it',
                List.of(new Parameter(':name', ParameterType.STRING, name)), Boolean, Results.&toExists)
    }

    /**
     * Create a connection string for the given database
     *
     * @param database The database to which the library should connect
     * @return The connection string for the database
     */
    private static String connString(String database) {
        return "jdbc:postgresql://localhost/$database?user=postgres&password=postgres"
    }

}
