package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

/**
 * PostgreSQL integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName('Groovy | PostgreSQL: Find')
final class PostgreSQLFindIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new PgDB().withCloseable FindFunctions.&allDefault
    }

    @Test
    @DisplayName('all sorts data ascending')
    void allAscending() {
        new PgDB().withCloseable FindFunctions.&allAscending
    }

    @Test
    @DisplayName('all sorts data descending')
    void allDescending() {
        new PgDB().withCloseable FindFunctions.&allDescending
    }

    @Test
    @DisplayName('all sorts data numerically')
    void allNumOrder() {
        new PgDB().withCloseable FindFunctions.&allNumOrder
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new PgDB().withCloseable FindFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new PgDB().withCloseable FindFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new PgDB().withCloseable FindFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new PgDB().withCloseable FindFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new PgDB().withCloseable FindFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new PgDB().withCloseable FindFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new PgDB().withCloseable FindFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new PgDB().withCloseable FindFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new PgDB().withCloseable FindFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains retrieves matching documents')
    void byContainsMatch() {
        new PgDB().withCloseable FindFunctions.&byContainsMatch
    }

    @Test
    @DisplayName('byContains retrieves ordered matching documents')
    void byContainsMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&byContainsMatchOrdered
    }

    @Test
    @DisplayName('byContains succeeds when no documents match')
    void byContainsNoMatch() {
        new PgDB().withCloseable FindFunctions.&byContainsNoMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves matching documents')
    void byJsonPathMatch() {
        new PgDB().withCloseable FindFunctions.&byJsonPathMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves ordered matching documents')
    void byJsonPathMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&byJsonPathMatchOrdered
    }

    @Test
    @DisplayName('byJsonPath succeeds when no documents match')
    void byJsonPathNoMatch() {
        new PgDB().withCloseable FindFunctions.&byJsonPathNoMatch
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new PgDB().withCloseable FindFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new PgDB().withCloseable FindFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new PgDB().withCloseable FindFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document')
    void firstByContainsMatchOne() {
        new PgDB().withCloseable FindFunctions.&firstByContainsMatchOne
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many')
    void firstByContainsMatchMany() {
        new PgDB().withCloseable FindFunctions.&firstByContainsMatchMany
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many (ordered)')
    void firstByContainsMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&firstByContainsMatchOrdered
    }

    @Test
    @DisplayName('firstByContains returns null when no document matches')
    void firstByContainsNoMatch() {
        new PgDB().withCloseable FindFunctions.&firstByContainsNoMatch
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document')
    void firstByJsonPathMatchOne() {
        new PgDB().withCloseable FindFunctions.&firstByJsonPathMatchOne
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many')
    void firstByJsonPathMatchMany() {
        new PgDB().withCloseable FindFunctions.&firstByJsonPathMatchMany
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many (ordered)')
    void firstByJsonPathMatchOrdered() {
        new PgDB().withCloseable FindFunctions.&firstByJsonPathMatchOrdered
    }

    @Test
    @DisplayName('firstByJsonPath returns null when no document matches')
    void firstByJsonPathNoMatch() {
        new PgDB().withCloseable FindFunctions.&firstByJsonPathNoMatch
    }
}
