package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Exists` object / `existsBy*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Exists')
final class SQLiteExistsIT {

    @Test
    @DisplayName('byId returns true when a document matches the ID')
    void byIdMatch() {
        new SQLiteDB().withCloseable ExistsFunctions.&byIdMatch
    }

    @Test
    @DisplayName('byId returns false when no document matches the ID')
    void byIdNoMatch() {
        new SQLiteDB().withCloseable ExistsFunctions.&byIdNoMatch
    }

    @Test
    @DisplayName('byFields returns true when documents match')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable ExistsFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields returns false when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable ExistsFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { ExistsFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { ExistsFunctions.byJsonPathMatch db }
        }
    }
}
