package solutions.bitbadger.documents.kotlinx

import kotlinx.serialization.json.Json

/**
 * Configuration for document serialization
 */
object DocumentConfig {

    val options = Json {
        coerceInputValues = true
        encodeDefaults    = true
        explicitNulls     = false
    }

    /**
     * Serialize a document to JSON
     *
     * @param document The document to be serialized
     * @return The JSON string with the serialized document
     */
    inline fun <reified TDoc> serialize(document: TDoc) =
        options.encodeToString(document)

    /**
     * Deserialize a document from JSON
     *
     * @param json The JSON string with the serialized document
     * @return The document created from the given JSON
     */
    inline fun <reified TDoc> deserialize(json: String) =
        options.decodeFromString<TDoc>(json)
}
