package solutions.bitbadger.documents.kotlinx.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.assertThrows
import solutions.bitbadger.documents.DocumentException
import kotlin.test.Test

/**
 * SQLite integration tests for the `Definition` object / `ensure*` connection extension functions
 */
@DisplayName("KotlinX | SQLite: Definition")
class SQLiteDefinitionIT {

    @Test
    @DisplayName("ensureTable creates table and index")
    fun ensureTable() =
        SQLiteDB().use(DefinitionFunctions::ensureTable)

    @Test
    @DisplayName("ensureFieldIndex creates an index")
    fun ensureFieldIndex() =
        SQLiteDB().use(DefinitionFunctions::ensureFieldIndex)

    @Test
    @DisplayName("ensureDocumentIndex fails for full index")
    fun ensureDocumentIndexFull() {
        assertThrows<DocumentException> { SQLiteDB().use(DefinitionFunctions::ensureDocumentIndexFull) }
    }

    @Test
    @DisplayName("ensureDocumentIndex fails for optimized index")
    fun ensureDocumentIndexOptimized() {
        assertThrows<DocumentException> { SQLiteDB().use(DefinitionFunctions::ensureDocumentIndexOptimized) }
    }
}
