package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{DisplayName, Test}
import solutions.bitbadger.documents.{DocumentException, Parameter, ParameterType}

@DisplayName("Scala | Parameter")
class ParameterTest:

  @Test
  @DisplayName("Construction with colon-prefixed name")
  def ctorWithColon(): Unit =
    val p = Parameter(":test", ParameterType.STRING, "ABC")
    assertEquals(":test", p.getName, "Parameter name was incorrect")
    assertEquals(ParameterType.STRING, p.getType, "Parameter type was incorrect")
    assertEquals("ABC", p.getValue, "Parameter value was incorrect")

  @Test
  @DisplayName("Construction with at-sign-prefixed name")
  def ctorWithAtSign(): Unit =
    val p = Parameter("@yo", ParameterType.NUMBER, null)
    assertEquals("@yo", p.getName, "Parameter name was incorrect")
    assertEquals(ParameterType.NUMBER, p.getType, "Parameter type was incorrect")
    assertNull(p.getValue, "Parameter value was incorrect")

  @Test
  @DisplayName("Construction fails with incorrect prefix")
  def ctorFailsForPrefix(): Unit =
    assertThrows(classOf[DocumentException], () => Parameter("it", ParameterType.JSON, ""))
