package solutions.bitbadger.documents.scala.tests

import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.{AfterEach, DisplayName, Test}
import solutions.bitbadger.documents.{DocumentException, Field, Parameter, ParameterType}
import solutions.bitbadger.documents.query.RemoveFieldsQuery

import scala.jdk.CollectionConverters.*

@DisplayName("Scala | Query | RemoveFieldsQuery")
class RemoveFieldsQueryTest:

  /**
   * Reset the dialect
   */
  @AfterEach
  def cleanUp(): Unit =
    ForceDialect.none()

  @Test
  @DisplayName("byId generates correctly | PostgreSQL")
  def byIdPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"UPDATE $TEST_TABLE SET data = data - :name::text[] WHERE data->>'id' = :id",
      RemoveFieldsQuery.byId(TEST_TABLE, List(Parameter(":name", ParameterType.STRING, "{a,z}")).asJava),
      "Remove Fields query not constructed correctly")

  @Test
  @DisplayName("byId generates correctly | SQLite")
  def byIdSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"UPDATE $TEST_TABLE SET data = json_remove(data, :name0, :name1) WHERE data->>'id' = :id",
      RemoveFieldsQuery.byId(TEST_TABLE,
        List(Parameter(":name0", ParameterType.STRING, "a"),Parameter(":name1", ParameterType.STRING, "z")).asJava),
      "Remove Field query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | PostgreSQL")
  def byFieldsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"UPDATE $TEST_TABLE SET data = data - :name::text[] WHERE data->>'f' > :g",
      RemoveFieldsQuery.byFields(TEST_TABLE, List(Parameter(":name", ParameterType.STRING, "{b,c}")).asJava,
        List(Field.greater("f", "", ":g")).asJava),
      "Remove Field query not constructed correctly")

  @Test
  @DisplayName("byFields generates correctly | SQLite")
  def byFieldsSQLite(): Unit =
    ForceDialect.sqlite()
    assertEquals(s"UPDATE $TEST_TABLE SET data = json_remove(data, :name0, :name1) WHERE data->>'f' > :g",
      RemoveFieldsQuery.byFields(TEST_TABLE,
        List(Parameter(":name0", ParameterType.STRING, "b"), Parameter(":name1", ParameterType.STRING, "c")).asJava,
        List(Field.greater("f", "", ":g")).asJava),
      "Remove Field query not constructed correctly")

  @Test
  @DisplayName("byContains generates correctly | PostgreSQL")
  def byContainsPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"UPDATE $TEST_TABLE SET data = data - :name::text[] WHERE data @> :criteria",
      RemoveFieldsQuery.byContains(TEST_TABLE, List(Parameter(":name", ParameterType.STRING, "{m,n}")).asJava),
      "Remove Field query not constructed correctly")

  @Test
  @DisplayName("byContains fails | SQLite")
  def byContainsSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => RemoveFieldsQuery.byContains(TEST_TABLE, List().asJava))

  @Test
  @DisplayName("byJsonPath generates correctly | PostgreSQL")
  def byJsonPathPostgres(): Unit =
    ForceDialect.postgres()
    assertEquals(s"UPDATE $TEST_TABLE SET data = data - :name::text[] WHERE jsonb_path_exists(data, :path::jsonpath)",
      RemoveFieldsQuery.byJsonPath(TEST_TABLE, List(Parameter(":name", ParameterType.STRING, "{o,p}")).asJava),
      "Remove Field query not constructed correctly")

  @Test
  @DisplayName("byJsonPath fails | SQLite")
  def byJsonPathSQLite(): Unit =
    ForceDialect.sqlite()
    assertThrows(classOf[DocumentException], () => RemoveFieldsQuery.byJsonPath(TEST_TABLE, List().asJava))
