package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

object CountFunctions:

  def all(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.countAll(TEST_TABLE), "There should have been 5 documents in the table")

  def byFieldsNumeric(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(3L, db.conn.countByFields(TEST_TABLE, Field.between("numValue", 10, 20) :: Nil),
      "There should have been 3 matching documents")

  def byFieldsAlpha(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(1L, db.conn.countByFields(TEST_TABLE, Field.between("value", "aardvark", "apple") :: Nil),
      "There should have been 1 matching document")

  def byContainsMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(2L, db.conn.countByContains(TEST_TABLE, Map.Map1("value", "purple")),
      "There should have been 2 matching documents")

  def byContainsNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(0L, db.conn.countByContains(TEST_TABLE, Map.Map1("value", "magenta")),
      "There should have been no matching documents")

  def byJsonPathMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(2L, db.conn.countByJsonPath(TEST_TABLE, "$.numValue ? (@ < 5)"),
      "There should have been 2 matching documents")

  def byJsonPathNoMatch(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(0L, db.conn.countByJsonPath(TEST_TABLE, "$.numValue ? (@ > 100)"),
      "There should have been no matching documents")
