package solutions.bitbadger.documents.scala.tests.integration

import solutions.bitbadger.documents.{Configuration, Parameter, ParameterType}
import solutions.bitbadger.documents.scala.Results
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE

import java.sql.Connection
import scala.util.Using

/**
 * A wrapper for a throwaway PostgreSQL database
 */
class PgDB extends ThrowawayDatabase:

  Configuration.setConnectionString(PgDB.connString("postgres"))
  Using(Configuration.dbConn()) { conn => conn.customNonQuery(s"CREATE DATABASE $dbName") }
  
  Configuration.setConnectionString(PgDB.connString(dbName))

  override val conn: Connection = Configuration.dbConn()

  conn.ensureTable(TEST_TABLE)

  override def close(): Unit =
    conn.close()
    Configuration.setConnectionString(PgDB.connString("postgres"))
    Using(Configuration.dbConn()) { conn => conn.customNonQuery(s"DROP DATABASE $dbName") }
    Configuration.setConnectionString(null)

  override def dbObjectExists(name: String): Boolean =
    conn.customScalar("SELECT EXISTS (SELECT 1 FROM pg_class WHERE relname = :name) AS it",
      Parameter(":name", ParameterType.STRING, name) :: Nil, Results.toExists)


object PgDB:

  /**
   * Create a connection string for the given database
   *
   * @param database The database to which the library should connect
   * @return The connection string for the database
   */
  private def connString(database: String): String =
    s"jdbc:postgresql://localhost/$database?user=postgres&password=postgres"
