package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}

import scala.util.Using

/**
 * PostgreSQL integration tests for the `Find` object / `find*` connection extension functions
 */
@DisplayName("Scala | PostgreSQL: Find")
class PostgreSQLFindIT:

  @Test
  @DisplayName("all retrieves all documents")
  def allDefault(): Unit =
    Using(PgDB()) { db => FindFunctions.allDefault(db) }

  @Test
  @DisplayName("all sorts data ascending")
  def allAscending(): Unit =
    Using(PgDB()) { db => FindFunctions.allAscending(db) }

  @Test
  @DisplayName("all sorts data descending")
  def allDescending(): Unit =
    Using(PgDB()) { db => FindFunctions.allDescending(db) }

  @Test
  @DisplayName("all sorts data numerically")
  def allNumOrder(): Unit =
    Using(PgDB()) { db => FindFunctions.allNumOrder(db) }

  @Test
  @DisplayName("all succeeds with an empty table")
  def allEmpty(): Unit =
    Using(PgDB()) { db => FindFunctions.allEmpty(db) }

  @Test
  @DisplayName("byId retrieves a document via a string ID")
  def byIdString(): Unit =
    Using(PgDB()) { db => FindFunctions.byIdString(db) }

  @Test
  @DisplayName("byId retrieves a document via a numeric ID")
  def byIdNumber(): Unit =
    Using(PgDB()) { db => FindFunctions.byIdNumber(db) }

  @Test
  @DisplayName("byId returns null when a matching ID is not found")
  def byIdNotFound(): Unit =
    Using(PgDB()) { db => FindFunctions.byIdNotFound(db) }

  @Test
  @DisplayName("byFields retrieves matching documents")
  def byFieldsMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields retrieves ordered matching documents")
  def byFieldsMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsMatchOrdered(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with a numeric IN clause")
  def byFieldsMatchNumIn(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsMatchNumIn(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
  def byFieldsMatchInArray(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsMatchInArray(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
  def byFieldsNoMatchInArray(): Unit =
    Using(PgDB()) { db => FindFunctions.byFieldsNoMatchInArray(db) }

  @Test
  @DisplayName("byContains retrieves matching documents")
  def byContainsMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byContainsMatch(db) }

  @Test
  @DisplayName("byContains retrieves ordered matching documents")
  def byContainsMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.byContainsMatchOrdered(db) }

  @Test
  @DisplayName("byContains succeeds when no documents match")
  def byContainsNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byContainsNoMatch(db) }

  @Test
  @DisplayName("byJsonPath retrieves matching documents")
  def byJsonPathMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byJsonPathMatch(db) }

  @Test
  @DisplayName("byJsonPath retrieves ordered matching documents")
  def byJsonPathMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.byJsonPathMatchOrdered(db) }

  @Test
  @DisplayName("byJsonPath succeeds when no documents match")
  def byJsonPathNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.byJsonPathNoMatch(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document")
  def firstByFieldsMatchOne(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByFieldsMatchOne(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many")
  def firstByFieldsMatchMany(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByFieldsMatchMany(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many (ordered)")
  def firstByFieldsMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("firstByFields returns null when no document matches")
  def firstByFieldsNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByFieldsNoMatch(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document")
  def firstByContainsMatchOne(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByContainsMatchOne(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document among many")
  def firstByContainsMatchMany(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByContainsMatchMany(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document among many (ordered)")
  def firstByContainsMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByContainsMatchOrdered(db) }

  @Test
  @DisplayName("firstByContains returns null when no document matches")
  def firstByContainsNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByContainsNoMatch(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document")
  def firstByJsonPathMatchOne(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByJsonPathMatchOne(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document among many")
  def firstByJsonPathMatchMany(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByJsonPathMatchMany(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
  def firstByJsonPathMatchOrdered(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByJsonPathMatchOrdered(db) }

  @Test
  @DisplayName("firstByJsonPath returns null when no document matches")
  def firstByJsonPathNoMatch(): Unit =
    Using(PgDB()) { db => FindFunctions.firstByJsonPathNoMatch(db) }
