package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.assertThrows
import org.junit.jupiter.api.{DisplayName, Test}
import solutions.bitbadger.documents.DocumentException

import scala.util.Using

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Scala | SQLite: Json")
class SQLiteJsonIT:

  @Test
  @DisplayName("all retrieves all documents")
  def allDefault(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.allDefault(db) }

  @Test
  @DisplayName("all succeeds with an empty table")
  def allEmpty(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.allEmpty(db) }

  @Test
  @DisplayName("byId retrieves a document via a string ID")
  def byIdString(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byIdString(db) }

  @Test
  @DisplayName("byId retrieves a document via a numeric ID")
  def byIdNumber(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byIdNumber(db) }

  @Test
  @DisplayName("byId returns null when a matching ID is not found")
  def byIdNotFound(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byIdNotFound(db) }

  @Test
  @DisplayName("byFields retrieves matching documents")
  def byFieldsMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields retrieves ordered matching documents")
  def byFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsMatchOrdered(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with a numeric IN clause")
  def byFieldsMatchNumIn(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsMatchNumIn(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
  def byFieldsMatchInArray(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsMatchInArray(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
  def byFieldsNoMatchInArray(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.byFieldsNoMatchInArray(db) }

  @Test
  @DisplayName("byContains fails")
  def byContainsFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => JsonFunctions.byContainsMatch(db)) }

  @Test
  @DisplayName("byJsonPath fails")
  def byJsonPathFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => JsonFunctions.byJsonPathMatch(db)) }

  @Test
  @DisplayName("firstByFields retrieves a matching document")
  def firstByFieldsMatchOne(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.firstByFieldsMatchOne(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many")
  def firstByFieldsMatchMany(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.firstByFieldsMatchMany(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many (ordered)")
  def firstByFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.firstByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("firstByFields returns null when no document matches")
  def firstByFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.firstByFieldsNoMatch(db) }

  @Test
  @DisplayName("firstByContains fails")
  def firstByContainsFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => JsonFunctions.firstByContainsMatchOne(db))
    }

  @Test
  @DisplayName("firstByJsonPath fails")
  def firstByJsonPathFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => JsonFunctions.firstByJsonPathMatchOne(db))
    }

  @Test
  @DisplayName("writeAll retrieves all documents")
  def writeAllDefault(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeAllDefault(db) }

  @Test
  @DisplayName("writeAll succeeds with an empty table")
  def writeAllEmpty(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeAllEmpty(db) }

  @Test
  @DisplayName("writeById retrieves a document via a string ID")
  def writeByIdString(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByIdString(db) }

  @Test
  @DisplayName("writeById retrieves a document via a numeric ID")
  def writeByIdNumber(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByIdNumber(db) }

  @Test
  @DisplayName("writeById returns null when a matching ID is not found")
  def writeByIdNotFound(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByIdNotFound(db) }

  @Test
  @DisplayName("writeByFields retrieves matching documents")
  def writeByFieldsMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsMatch(db) }

  @Test
  @DisplayName("writeByFields retrieves ordered matching documents")
  def writeByFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("writeByFields retrieves matching documents with a numeric IN clause")
  def writeByFieldsMatchNumIn(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsMatchNumIn(db) }

  @Test
  @DisplayName("writeByFields succeeds when no documents match")
  def writeByFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsNoMatch(db) }

  @Test
  @DisplayName("writeByFields retrieves matching documents with an IN_ARRAY comparison")
  def writeByFieldsMatchInArray(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsMatchInArray(db) }

  @Test
  @DisplayName("writeByFields succeeds when no documents match an IN_ARRAY comparison")
  def writeByFieldsNoMatchInArray(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeByFieldsNoMatchInArray(db) }

  @Test
  @DisplayName("writeByContains fails")
  def writeByContainsFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => JsonFunctions.writeByContainsMatch(db)) }

  @Test
  @DisplayName("writeByJsonPath fails")
  def writeByJsonPathFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => JsonFunctions.writeByJsonPathMatch(db)) }

  @Test
  @DisplayName("writeFirstByFields retrieves a matching document")
  def writeFirstByFieldsMatchOne(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeFirstByFieldsMatchOne(db) }

  @Test
  @DisplayName("writeFirstByFields retrieves a matching document among many")
  def writeFirstByFieldsMatchMany(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeFirstByFieldsMatchMany(db) }

  @Test
  @DisplayName("writeFirstByFields retrieves a matching document among many (ordered)")
  def writeFirstByFieldsMatchOrdered(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeFirstByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("writeFirstByFields returns null when no document matches")
  def writeFirstByFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => JsonFunctions.writeFirstByFieldsNoMatch(db) }

  @Test
  @DisplayName("writeFirstByContains fails")
  def writeFirstByContainsFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => JsonFunctions.writeFirstByContainsMatchOne(db))
    }

  @Test
  @DisplayName("writeFirstByJsonPath fails")
  def writeFirstByJsonPathFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => JsonFunctions.writeFirstByJsonPathMatchOne(db))
    }
