package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}
import org.junit.jupiter.api.Assertions.assertThrows
import solutions.bitbadger.documents.DocumentException

import scala.util.Using

/**
 * SQLite integration tests for the `Patch` object / `patchBy*` connection extension functions
 */
@DisplayName("Scala | SQLite: Patch")
class SQLitePatchIT:

  @Test
  @DisplayName("byId patches an existing document")
  def byIdMatch(): Unit =
    Using(SQLiteDB()) { db => PatchFunctions.byIdMatch(db) }

  @Test
  @DisplayName("byId succeeds for a non-existent document")
  def byIdNoMatch(): Unit =
    Using(SQLiteDB()) { db => PatchFunctions.byIdNoMatch(db) }

  @Test
  @DisplayName("byFields patches matching document")
  def byFieldsMatch(): Unit =
    Using(SQLiteDB()) { db => PatchFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => PatchFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byContains fails")
  def byContainsFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => PatchFunctions.byContainsMatch(db)) }
  
  @Test
  @DisplayName("byJsonPath fails")
  def byJsonPathFails(): Unit =
    Using(SQLiteDB()) { db => assertThrows(classOf[DocumentException], () => PatchFunctions.byJsonPathMatch(db)) }

