package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}
import org.junit.jupiter.api.Assertions.assertThrows
import solutions.bitbadger.documents.DocumentException

import scala.util.Using

/**
 * SQLite integration tests for the `RemoveFields` object / `removeFieldsBy*` connection extension functions
 */
@DisplayName("Scala | SQLite: RemoveFields")
class SQLiteRemoveFieldsIT:

  @Test
  @DisplayName("byId removes fields from an existing document")
  def byIdMatchFields(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byIdMatchFields(db) }

  @Test
  @DisplayName("byId succeeds when fields do not exist on an existing document")
  def byIdMatchNoFields(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byIdMatchNoFields(db) }

  @Test
  @DisplayName("byId succeeds when no document exists")
  def byIdNoMatch(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byIdNoMatch(db) }

  @Test
  @DisplayName("byFields removes fields from matching documents")
  def byFieldsMatchFields(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byFieldsMatchFields(db) }

  @Test
  @DisplayName("byFields succeeds when fields do not exist on matching documents")
  def byFieldsMatchNoFields(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byFieldsMatchNoFields(db) }

  @Test
  @DisplayName("byFields succeeds when no matching documents exist")
  def byFieldsNoMatch(): Unit =
    Using(SQLiteDB()) { db => RemoveFieldsFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byContains fails")
  def byContainsFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => RemoveFieldsFunctions.byContainsMatchFields(db))
    }

  @Test
  @DisplayName("byJsonPath fails")
  def byJsonPathFails(): Unit =
    Using(SQLiteDB()) { db =>
      assertThrows(classOf[DocumentException], () => RemoveFieldsFunctions.byJsonPathMatchFields(db))
    }
