package solutions.bitbadger.documents.scala.tests.integration

import solutions.bitbadger.documents.AutoId
import solutions.bitbadger.documents.java.DocumentConfig

import java.sql.Connection

/**
 * Common trait for PostgreSQL and SQLite throwaway databases
 */
trait ThrowawayDatabase extends AutoCloseable:

  /** The database connection for the throwaway database */
  def conn: Connection

  /**
   * Determine if a database object exists
   *
   * @param name The name of the object whose existence should be checked
   * @return True if the object exists, false if not
   */
  def dbObjectExists(name: String): Boolean

  /** The name for the throwaway database */
  val dbName = s"throwaway_${AutoId.generateRandomString(8)}"

  // Use a Jackson-based document serializer for testing
  DocumentConfig.setSerializer(JacksonDocumentSerializer())
